import abc


class Tokenizer():
    """
    Abstract class for splitting and tokenizing raw text.
    """

    def add_matcher(self, matcher):
        """
        Add the match object on the Tokenizer.

        Args:
            matcher: dmf.match.matchers.Matcher

        Returns: None

        """
        self.matcher = matcher
        """The matcher object"""

    @abc.abstractmethod
    def tokenize_record(self, record):
        """
        An abstract method to tokenize record's data.

        Returns: List

        """
        return

    def tokenize(self):
        """
        Get each data by for loop and call method tokenize_record().
        Returns a dictionary with feature as key and list tokens as value.

        Returns: Dict

        """
        dict_of_list_token = {}
        for feature, records in self.matcher.records_join.items():
            list_token = []
            for record in records.values():
                list_token.append(self.tokenize_string(record))
            dict_of_list_token[feature] = list_token
        return dict_of_list_token

    @abc.abstractmethod
    def normalize_record(self, record):
        """
        An abstract method to normalize record's data.
        """
        return record

    def normalize(self, records):
        """
        Get each data by for loop and call method normalize_record()

        Returns: List

        """
        nomalized_records = []
        for record in records:
            nomalized_records.append(self.normalize_record(record))
        return nomalized_records


class GenericTokenizer(Tokenizer):
    """
    A generic tokenizer use by calling function.
    """

    def __init__(self, splitter):
        self.splitter = splitter
        """Splitter function"""

    def tokenize_record(self, record):
        """
        A function that takes a string as input and returns a list/iterator of tokenized string items

        Args:
            record: String

        Returns: List

        """
        return self.splitter(record)



import abc
from dmf.structures.data import (
    Dataset,
)


class Reader():
    """
    Abstract class for reading data in dataset format.
    """

    @abc.abstractmethod
    def __init__(self):
        """

        Args:
            dataset: Dataset
        """
        self.dataset = Dataset()

    @abc.abstractmethod
    def read_func(self):
        """
        Function to read dataset.
        """
        return

    def read(self):
        """
        Method to read dataset

        Returns: Dataset
        """
        self.dataset.df = self.read_func()
        return self.dataset


class FileReader(Reader):
    """
    Class reader for read data from file.
    """

    def __init__(self, file_name):
        """

        Args:
            file_name: String
        """
        super().__init__()
        self.file_name = file_name
        """Name of file that contains a dataset"""


class DBReader(Reader):
    """
    Class reader for read data from database.
    """

    def __init__(self, db_host, db_user, db_passwd, db_name, sql):
        """

        Args:
            db_host: String
            db_user: String
            db_passwd: String
            db_name: String
        """
        super().__init__()
        self.db_host = db_host
        """Database host name to connect"""
        self.db_user = db_user
        """Database user name to connect"""
        self.db_passwd = db_passwd
        """Database password to connect"""
        self.db_name = db_name
        """Database name to connect"""
        self.sql = sql
        """Query to select data from MySQL database"""
        # self.connect()

    @abc.abstractmethod
    def connect(self):
        """
        Method for create a connection to the database.
        """
        return



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dm_framework", # It should be unique name
    version="0.2.0",
    description="The data matching framework for two dataset",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Phan Quoc Toan",
    author_email="quoctoanpk2511@gmail.com",
    url="https://github.com/quoctoanpk2511/matching-framework",
    project_urls={
        'Homepage': 'https://github.com/quoctoanpk2511/matching-framework',
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.8',
    install_requires=[
        "numpy==1.19.3"
        "pandas==1.1.5"
    ],
    entry_points={
        'console_scripts': [
            'git-tagup=git_tagup.__main__:main',
            'gtu=git_tagup.__main__:main',
        ],
    },
)



import abc
import numpy as np


class SimilarityScorer:
    """
    Abstract class for compute the similarity of the entities.
    """

    def add_matcher(self, matcher):
        """
                Add the match object on the SimilarityScorer.

                Args:
                    matcher: dmf.match.matchers.Matcher

                Returns: None

                """
        self.matcher = matcher
        """The matcher object"""

    @abc.abstractmethod
    def similarity_score(self, vector_matrix):
        """
        Compute similarity.

        Returns:
        -------
        similarity_matrix : dict
            An arary with shape of input array
        """

    def score(self):
        """
        Get each vectorized matrix by for loop and calling similarity_score().
        Normalize matrixes by calling normalize_matrixes().
        """
        similarity_dict = {}
        for feature, vector_matrix in self.matcher.vectorized_dict.items():
            similarity_dict[feature] = self.similarity_score(vector_matrix)
        self.matcher.similarity_matrix = self.normalize_matrixes(similarity_dict)

    def normalize_matrixes(self, similarity_dict):
        """
        Normalize similarity matrixes into one with it's weight.

        Args:
            similarity_dict: dict

        Returns:
        -------
        similarity_matrix: array
            Similarity matrix of records.
        """
        shape = self.matcher.get_count_records()
        similarity_matrix = np.zeros((shape, shape))
        for feature, distance in similarity_dict.items():
            similarity_matrix += similarity_dict[feature] * feature.weight
        return similarity_matrix
